// Copyright 2024 Matthew Rich <matthewrich.conf@gmail.com>. All rights reserved.

package codec

import (
_	"fmt"
	"github.com/stretchr/testify/assert"
_	"log"
	"testing"
	"strings"
	"encoding/json"
)

type TestDec struct {
	FormatType Format `yaml:"formattype" json:"formattype"`
}

func TestFormatType(t *testing.T) {
	yamlData := `
formattype: json
`
	v := &TestDec{}

	dec := NewYAMLStringDecoder(yamlData)
	e := dec.Decode(v)

	assert.Nil(t, e)

	assert.Equal(t, FormatJson, v.FormatType)
}

func TestFormatTypeErr(t *testing.T) {
	yamlData := `
formattype: foo
`

	v := &TestDec{}

	dec := NewYAMLStringDecoder(yamlData)
	e := dec.Decode(v)

	assert.ErrorIs(t, ErrInvalidFormat, e)
}

func TestFormatValidate(t *testing.T) {
	f := FormatYaml
	assert.Nil(t, f.Validate())

	var fail Format = Format("foo")
	assert.ErrorIs(t, fail.Validate(), ErrInvalidFormat)

	var testFormatSet Format
	assert.Nil(t, testFormatSet.Set("yaml"))

	assert.ErrorIs(t, testFormatSet.Set("yamlv3"), ErrInvalidFormat)
}

func TestFormatCodec(t *testing.T) {
	var output map[string]Format = make(map[string]Format)
	var writer strings.Builder
	encoder := FormatYaml.Encoder(&writer)
	assert.NotNil(t, encoder)

	decoder := FormatYaml.Decoder(strings.NewReader("formattype: json"))
	assert.Nil(t, decoder.Decode(output))
	assert.Equal(t, FormatJson, output["formattype"])
}

func TestFormatUnmarshal(t *testing.T) {
	var f Format 
	assert.Nil(t, json.Unmarshal([]byte("\"yaml\""), &f))
	assert.Equal(t, FormatYaml, f)
	assert.NotNil(t, json.Unmarshal([]byte("\"yaml"), &f))

	assert.Nil(t, json.Unmarshal([]byte("\"yml\""), &f))
	assert.Equal(t, FormatYaml, f)
}
