// Copyright 2024 Matthew Rich <matthewrich.conf@gmail.com>. All rights reserved.

package folio

import (
	"net/url"
	"decl/internal/transport"
	"decl/internal/data"
	"decl/internal/identifier"
	"errors"
	"strings"
)


var (
	ErrInvalidURI error = errors.New("Invalid URI")
)

type URI identifier.ID

func NewURI(u *url.URL) URI {
	return URI(u.String())
}

func (u URI) NewResource(document data.Document) (newResource data.Resource, err error) {
	if document == nil {
		declaration := NewDeclaration()
		if err = declaration.NewResource((*string)(&u)); err == nil {
			return declaration.Attributes, err
		}
	} else {
		newResource, err = document.NewResource(string(u))
		return
	}
	return
}

func (u URI) Parse() *url.URL {
	url, e := url.Parse(string(u))
	if e == nil {
		return url
	}
	return nil
}

func (u URI) Exists() bool {
	return transport.ExistsURI(string(u))
}

func (u URI) ContentReaderStream() (*transport.Reader, error) {
	return transport.NewReaderURI(string(u))
}

func (u URI) ContentWriterStream() (*transport.Writer, error) {
	return transport.NewWriterURI(string(u))
}

func (u URI) String() string {
	return string(u)
}

func (u *URI) SetURL(url *url.URL) {
	(*identifier.ID)(u).SetURL(url)
}

func (u URI) Extension() (string, string) {
	return (identifier.ID)(u).Extension()
}

func (u URI) ContentType() string {
	var ext strings.Builder
	exttype, fileext := u.Extension()
	if fileext == "" {
		return exttype
	}
	ext.WriteString(exttype)
	ext.WriteRune('.')
	ext.WriteString(fileext)
	return ext.String()
}

func (u URI) IsEmpty() bool {
	return (identifier.ID)(u).IsEmpty()
}
