// Copyright 2024 Matthew Rich <matthewrich.conf@gmail.com>. All rights reserved.

package types

import (
_	"context"
_	"encoding/json"
	"github.com/stretchr/testify/assert"
	"net/url"
	"testing"
)

func TestNewSourceTypes(t *testing.T) {
	testTypes := New[int]()
	assert.NotNil(t, testTypes)
}

func TestNewSourceTypesRegister(t *testing.T) {

	testTypes := New[int]()
	assert.NotNil(t, testTypes)

	testTypes.Register([]string{"foo"}, func(*url.URL) int { return 100 })

	r, e := testTypes.New("foo://")
	assert.Nil(t, e)
	assert.Equal(t, 100, r)
}

func TestTypesFromURI(t *testing.T) {
	testTypes := New[string]()
	assert.NotNil(t, testTypes)

	testTypes.Register([]string{"foo"}, func(*url.URL) string { return "test" })

	r, e := testTypes.New("foo://bar")
	assert.Nil(t, e)
	assert.Equal(t, "test", r)
}

func TestTypesHasType(t *testing.T) {
	testTypes := New[string]()
	assert.NotNil(t, testTypes)

	testTypes.Register([]string{"foo"}, func(*url.URL) string { return "hastest" })

	assert.True(t, testTypes.Has("foo"))
}

/*
func TestTypeName(t *testing.T) {
	testTypes.Register([]string{"file"}, func(*url.URL) string { return "typetest" })

	type fDocSourceName struct {
		Name TypeName `json:"type"`
	}
	fTypeName := &fDocSourceName{}
	jsonType := `{ "type": "file" }`
	e := json.Unmarshal([]byte(jsonType), &fTypeName)
	assert.Nil(t, e)
	assert.Equal(t, "file", string(fTypeName.Name))
}
*/
